#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # PIG et Paramètres sur Cloudera - énoncé
# 
# Utilisation de job paramétrables en Map/Reduce avec [PIG](https://pig.apache.org/) sur [Cloudera](https://www.cloudera.com/).

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Paramètres
# 
# Les sites web produisent des données en continu. On utilise fréquemment le même script pour traiter les donnéesd'un jour, du lendemain, de jour d'après... Tous les jours, on veut récupérer la fréquentation de la veille. La seule chose qui change est la date des données qu'on veut traiter. Plutôt que de recopier un script en entier pour changer une date qui apparaît parfois à plusieurs endroits, il est préférable d'écrire un script ou la date apparaît comme une variable.
# 
# Ce notebook va illustrer ce procédé sur la construction d'un histogramme. Le paramètre du script sera la largeur des barres de l'histogramme (ou [bin](http://en.wikipedia.org/wiki/Histogram#Number_of_bins_and_width) en anglais).

# ## Connexion au cluster
# 
# On prend le cluster [Cloudera](http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx/notebooks/td3a_cenonce_session6.html#p2). Il faut exécuter ce script pour pouvoir notifier au notebook que la variable ``params`` existe.

# In[2]:


import pyensae, pyquickhelper
from pyquickhelper.ipythonhelper import open_html_form
params={"server":"df...fr", "username":"", "password":""}
open_html_form(params=params,title="server + credentials", key_save="params")


# In[3]:


import pyensae
get_ipython().run_line_magic('load_ext', 'pyensae')
get_ipython().run_line_magic('load_ext', 'pyenbc')
password = params["password"]
server = params["server"]
username = params["username"]
client = get_ipython().run_line_magic('remote_open', '')
client


# ## Upload version
# 
# On commence par simuler des données.

# In[4]:


import random
with open("random.sample.txt", "w") as f :
    for i in range(0,10000) :
        x = random.random()
        f.write(str(x)+"\n")


# On créé un répertoire sur le cluster :

# In[5]:


get_ipython().run_line_magic('dfs_mkdir', 'random')


# In[6]:


get_ipython().run_line_magic('remote_up_cluster', 'random.sample.txt random/random.sample.txt')


# ## PIG et paramètres

# On indique un paramètre par le symbole : ``$bins``. La valeur du paramètre est passé sous forme de chaîne de caractères au script et remplacée telle quelle dans le script. Il en va de même des constantes déclarées grâce au mot-clé [%declare](https://pig.apache.org/docs/r0.11.1/cont.html#Examples-N1060D).
# 
# La sortie du script inclut le paramètre : cela permet de retrouver comment ces données ont été générées.

# In[7]:


get_ipython().run_cell_magic('PIG', 'histogram.pig', "\nvalues = LOAD 'random/random.sample.txt' USING PigStorage('\\t') AS (x:double);\n\nvalues_h = FOREACH values GENERATE x, ((int)(x / $bins)) * $bins AS h ;\n\nhist_group = GROUP values_h BY h ;\n\nhist = FOREACH hist_group GENERATE group, COUNT(values_h) AS nb ;\n\nSTORE hist INTO 'random/histo_$bins.txt' USING PigStorage('\\t') ;\n")


# Pour supprimer les précédents résultats :

# In[8]:


if client.dfs_exists("random/histo_0.1.txt"):
    client.dfs_rm("random/histo_0.1.txt", recursive=True)


# On exécute le job. Comme la commande magique supportant les paramètres n'existe pas encore, il faut utiliser la variable ``client`` et sa méthode [pig_submit](http://www.xavierdupre.fr/app/pyensae/helpsphinx/pyensae/remote/ssh_remote_connection.html?highlight=pig_submit#pyensae.remote.ssh_remote_connection.ASSHClient.pig_submit) qui fait la même chose. Elle upload le script puis le soumet.

# In[9]:


client.pig_submit("histogram.pig", redirection="redirection", params =dict(bins="0.1") )


# S'il se produit des erreurs, il est recommandé d'afficher plus de lignes :

# In[10]:


get_ipython().run_line_magic('remote_cmd', 'tail redirection.err')


# On vérifie que tout s'est bien passé. La taille devrait être équivalent à l'entrée.

# In[11]:


get_ipython().run_line_magic('dfs_ls', 'random')


# In[12]:


if os.path.exists("histo.txt") : os.remove("histo.txt")
client.download_cluster("random/histo_0.1.txt","histo.txt", merge=True)


# In[13]:


import matplotlib.pyplot as plt
plt.style.use('ggplot')
import pandas
df = pandas.read_csv("histo.txt", sep="\t",names=["bin","nb"])
df.plot(x="bin", y="nb", kind="bar")


# ## Exercice 1 : min, max
# 
# Ajouter deux paramètres pour construire l'histogramme entre deux valeurs ``a``,``b``.

# In[14]: